@@ -225,12 +225,210 @@ func TestFloatingIPFromSchema(t *testing.T) {
 	})
 }
 
+func TestPrimaryIPFromSchema(t *testing.T) {
+	t.Run("IPv6", func(t *testing.T) {
+		data := []byte(`{
+		"assignee_id": 17,
+		"assignee_type": "server",
+		"auto_delete": true,
+		"blocked": true,
+		"created": "2017-08-16T17:29:14+00:00",
+		"datacenter": {
+			"description": "Falkenstein DC Park 8",
+			"id": 42,
+			"location": {
+				"city": "Falkenstein",
+				"country": "DE",
+				"description": "Falkenstein DC Park 1",
+				"id": 1,
+				"latitude": 50.47612,
+				"longitude": 12.370071,
+				"name": "fsn1",
+				"network_zone": "eu-central"
+			},
+			"name": "fsn1-dc8",
+			"server_types": {
+				"available": [],
+				"available_for_migration": [],
+				"supported": []
+			}
+		},
+		"dns_ptr": [
+			{
+				"dns_ptr": "server.example.com",
+				"ip": "fe80::"
+			}
+		],
+		"id": 4711,
+		"ip": "fe80::/64",
+		"labels": {
+			"key": "value",
+			"key2": "value2"
+		},
+		"name": "Web Frontend",
+		"protection": {
+			"delete": true
+		},
+		"type": "ipv6"
+  }`)
+
+		var s schema.PrimaryIP
+		if err := json.Unmarshal(data, &s); err != nil {
+			t.Fatal(err)
+		}
+		primaryIP := PrimaryIPFromSchema(s)
+
+		if primaryIP.ID != 4711 {
+			t.Errorf("unexpected ID: %v", primaryIP.ID)
+		}
+		if !primaryIP.Blocked {
+			t.Errorf("unexpected value for Blocked: %v", primaryIP.Blocked)
+		}
+		if !primaryIP.AutoDelete {
+			t.Errorf("unexpected value for AutoDelete: %v", primaryIP.AutoDelete)
+		}
+		if primaryIP.Name != "Web Frontend" {
+			t.Errorf("unexpected name: %v", primaryIP.Name)
+		}
+
+		if primaryIP.IP.String() != "fe80::" {
+			t.Errorf("unexpected IP: %v", primaryIP.IP)
+		}
+		if primaryIP.Type != PrimaryIPTypeIPv6 {
+			t.Errorf("unexpected Type: %v", primaryIP.Type)
+		}
+		if primaryIP.AssigneeType != "server" {
+			t.Errorf("unexpected AssigneeType: %v", primaryIP.AssigneeType)
+		}
+		if primaryIP.AssigneeID != 17 {
+			t.Errorf("unexpected AssigneeID: %v", primaryIP.AssigneeID)
+		}
+		dnsPTR, err := primaryIP.GetDNSPtrForIP(primaryIP.IP)
+		if err != nil {
+			t.Fatal(err)
+		}
+		if primaryIP.DNSPtr == nil || dnsPTR == "" {
+			t.Errorf("unexpected DNS ptr: %v", primaryIP.DNSPtr)
+		}
+		if primaryIP.Datacenter.Name != "fsn1-dc8" {
+			t.Errorf("unexpected datacenter: %v", primaryIP.Datacenter)
+		}
+		if !primaryIP.Protection.Delete {
+			t.Errorf("unexpected Protection.Delete: %v", primaryIP.Protection.Delete)
+		}
+		if primaryIP.Labels["key"] != "value" || primaryIP.Labels["key2"] != "value2" {
+			t.Errorf("unexpected Labels: %v", primaryIP.Labels)
+		}
+		if !primaryIP.Created.Equal(time.Date(2017, 8, 16, 17, 29, 14, 0, time.UTC)) {
+			t.Errorf("unexpected created date: %v", primaryIP.Created)
+		}
+	})
+	t.Run("IPv4", func(t *testing.T) {
+		data := []byte(`{
+		"assignee_id": 17,
+		"assignee_type": "server",
+		"auto_delete": true,
+		"blocked": true,
+		"created": "2017-08-16T17:29:14+00:00",
+		"datacenter": {
+			"description": "Falkenstein DC Park 8",
+			"id": 42,
+			"location": {
+				"city": "Falkenstein",
+				"country": "DE",
+				"description": "Falkenstein DC Park 1",
+				"id": 1,
+				"latitude": 50.47612,
+				"longitude": 12.370071,
+				"name": "fsn1",
+				"network_zone": "eu-central"
+			},
+			"name": "fsn1-dc8",
+			"server_types": {
+				"available": [],
+				"available_for_migration": [],
+				"supported": []
+			}
+		},
+		"dns_ptr": [
+			{
+				"dns_ptr": "server.example.com",
+				"ip": "127.0.0.1"
+			}
+		],
+		"id": 4711,
+		"ip": "127.0.0.1",
+		"labels": {
+			"key": "value",
+			"key2": "value2"
+		},
+		"name": "Web Frontend",
+		"protection": {
+			"delete": true
+		},
+		"type": "ipv4"
+  }`)
+
+		var s schema.PrimaryIP
+		if err := json.Unmarshal(data, &s); err != nil {
+			t.Fatal(err)
+		}
+		primaryIP := PrimaryIPFromSchema(s)
+
+		if primaryIP.ID != 4711 {
+			t.Errorf("unexpected ID: %v", primaryIP.ID)
+		}
+		if !primaryIP.Blocked {
+			t.Errorf("unexpected value for Blocked: %v", primaryIP.Blocked)
+		}
+		if !primaryIP.AutoDelete {
+			t.Errorf("unexpected value for AutoDelete: %v", primaryIP.AutoDelete)
+		}
+		if primaryIP.Name != "Web Frontend" {
+			t.Errorf("unexpected name: %v", primaryIP.Name)
+		}
+
+		if primaryIP.IP.String() != "127.0.0.1" {
+			t.Errorf("unexpected IP: %v", primaryIP.IP)
+		}
+		if primaryIP.Type != PrimaryIPTypeIPv4 {
+			t.Errorf("unexpected Type: %v", primaryIP.Type)
+		}
+		if primaryIP.AssigneeType != "server" {
+			t.Errorf("unexpected AssigneeType: %v", primaryIP.AssigneeType)
+		}
+		if primaryIP.AssigneeID != 17 {
+			t.Errorf("unexpected AssigneeID: %v", primaryIP.AssigneeID)
+		}
+		dnsPTR, err := primaryIP.GetDNSPtrForIP(primaryIP.IP)
+		if err != nil {
+			t.Fatal(err)
+		}
+		if primaryIP.DNSPtr == nil || dnsPTR == "" {
+			t.Errorf("unexpected DNS ptr: %v", primaryIP.DNSPtr)
+		}
+		if primaryIP.Datacenter.Name != "fsn1-dc8" {
+			t.Errorf("unexpected datacenter: %v", primaryIP.Datacenter)
+		}
+		if !primaryIP.Protection.Delete {
+			t.Errorf("unexpected Protection.Delete: %v", primaryIP.Protection.Delete)
+		}
+		if primaryIP.Labels["key"] != "value" || primaryIP.Labels["key2"] != "value2" {
+			t.Errorf("unexpected Labels: %v", primaryIP.Labels)
+		}
+		if !primaryIP.Created.Equal(time.Date(2017, 8, 16, 17, 29, 14, 0, time.UTC)) {
+			t.Errorf("unexpected created date: %v", primaryIP.Created)
+		}
+	})
+}
+
 func TestISOFromSchema(t *testing.T) {
 	data := []byte(`{
 		"id": 4711,
 		"name": "FreeBSD-11.0-RELEASE-amd64-dvd1",
 		"description": "FreeBSD 11.0 x64",
 		"type": "public",
+		"architecture": "x86",
 		"deprecated": "2018-02-28T00:00:00+00:00"
 	}`)
 
@@ -251,6 +449,11 @@ func TestISOFromSchema(t *testing.T) {
 	if iso.Type != ISOTypePublic {
 		t.Errorf("unexpected type: %v", iso.Type)
 	}
+	if iso.Architecture == nil {
+		t.Errorf("unexpected empty architecture")
+	} else if *iso.Architecture != ArchitectureX86 {
+		t.Errorf("unexpected architecture: %s", *iso.Architecture)
+	}
 	if iso.Deprecated.IsZero() {
 		t.Errorf("unexpected value for deprecated: %v", iso.Deprecated)
 	}
@@ -359,11 +562,7 @@ func TestServerFromSchema(t *testing.T) {
 		"status": "running",
 		"created": "2017-08-16T17:29:14+00:00",
 		"public_net": {
-			"ipv4": {
-				"ip": "1.2.3.4",
-				"blocked": false,
-				"dns_ptr": "server01.example.com"
-			},
+			"ipv4": null,
 			"ipv6": {
 				"ip": "2a01:4f8:1c11:3400::/64",
 				"blocked": false,
@@ -475,8 +674,11 @@ func TestServerFromSchema(t *testing.T) {
 	if !server.Created.Equal(time.Date(2017, 8, 16, 17, 29, 14, 0, time.UTC)) {
 		t.Errorf("unexpected created date: %v", server.Created)
 	}
-	if server.PublicNet.IPv4.IP.String() != "1.2.3.4" {
-		t.Errorf("unexpected public net IPv4 IP: %v", server.PublicNet.IPv4.IP)
+	if !server.PublicNet.IPv4.IsUnspecified() {
+		t.Errorf("unexpected public net IPv4: %v", server.PublicNet.IPv4)
+	}
+	if server.PublicNet.IPv6.IP.String() != "2a01:4f8:1c11:3400::" {
+		t.Errorf("unexpected public net IPv6 IP: %v", server.PublicNet.IPv6.IP)
 	}
 	if server.ServerType.ID != 2 {
 		t.Errorf("unexpected server type ID: %v", server.ServerType.ID)
@@ -577,11 +779,13 @@ func TestServerFromSchemaNoTraffic(t *testing.T) {
 func TestServerPublicNetFromSchema(t *testing.T) {
 	data := []byte(`{
 		"ipv4": {
+			"id": 1,
 			"ip": "1.2.3.4",
 			"blocked": false,
 			"dns_ptr": "server.example.com"
 		},
 		"ipv6": {
+			"id": 2,
 			"ip": "2a01:4f8:1c19:1403::/64",
 			"blocked": false,
 			"dns_ptr": []
@@ -600,10 +804,15 @@ func TestServerPublicNetFromSchema(t *testing.T) {
 		t.Fatal(err)
 	}
 	publicNet := ServerPublicNetFromSchema(s)
-
+	if publicNet.IPv4.ID != 1 {
+		t.Errorf("unexpected IPv4 ID: %v", publicNet.IPv4.ID)
+	}
 	if publicNet.IPv4.IP.String() != "1.2.3.4" {
 		t.Errorf("unexpected IPv4 IP: %v", publicNet.IPv4.IP)
 	}
+	if publicNet.IPv6.ID != 2 {
+		t.Errorf("unexpected IPv6 ID: %v", publicNet.IPv6.ID)
+	}
 	if publicNet.IPv6.Network.String() != "2a01:4f8:1c19:1403::/64" {
 		t.Errorf("unexpected IPv6 IP: %v", publicNet.IPv6.IP)
 	}
@@ -711,6 +920,8 @@ func TestServerTypeFromSchema(t *testing.T) {
 		"disk": 20,
 		"storage_type": "local",
 		"cpu_type": "shared",
+		"architecture": "x86",
+		"deprecation": null,
 		"prices": [
 			{
 				"location": "fsn1",
@@ -756,6 +967,12 @@ func TestServerTypeFromSchema(t *testing.T) {
 	if serverType.CPUType != CPUTypeShared {
 		t.Errorf("unexpected cpu type: %q", serverType.CPUType)
 	}
+	if serverType.Architecture != ArchitectureX86 {
+		t.Errorf("unexpected cpu architecture: %q", serverType.Architecture)
+	}
+	if serverType.Deprecation != nil {
+		t.Errorf("unexpected deprecation: %v", serverType.Deprecation)
+	}
 	if len(serverType.Pricings) != 1 {
 		t.Errorf("unexpected number of pricings: %d", len(serverType.Pricings))
 	} else {
@@ -936,6 +1153,7 @@ func TestImageFromSchema(t *testing.T) {
 		"bound_to": 1,
 		"os_flavor": "ubuntu",
 		"os_version": "16.04",
+		"architecture": "arm",
 		"rapid_deploy": false,
 		"protection": {
 			"delete": true
@@ -993,6 +1211,9 @@ func TestImageFromSchema(t *testing.T) {
 	if image.OSFlavor != "ubuntu" {
 		t.Errorf("unexpected OSFlavor: %v", image.OSFlavor)
 	}
+	if image.Architecture != ArchitectureARM {
+		t.Errorf("unexpected architecture: %v", image.Architecture)
+	}
 	if image.RapidDeploy {
 		t.Errorf("unexpected RapidDeploy: %v", image.RapidDeploy)
 	}
@@ -1731,9 +1952,9 @@ func TestCertificateFromSchema(t *testing.T) {
 				Name:           "my website cert",
 				Type:           "uploaded",
 				Certificate:    "-----BEGIN CERTIFICATE-----\n...",
-				Created:        mustParseTime(t, apiTimestampFormat, "2016-01-30T23:50:00+00:00"),
-				NotValidBefore: mustParseTime(t, apiTimestampFormat, "2016-01-30T23:51:00+00:00"),
-				NotValidAfter:  mustParseTime(t, apiTimestampFormat, "2016-01-30T23:55:00+00:00"),
+				Created:        mustParseTime(t, "2016-01-30T23:50:00+00:00"),
+				NotValidBefore: mustParseTime(t, "2016-01-30T23:51:00+00:00"),
+				NotValidAfter:  mustParseTime(t, "2016-01-30T23:55:00+00:00"),
 				DomainNames:    []string{"example.com", "webmail.example.com", "www.example.com"},
 				Fingerprint:    "03:c7:55:9b:2a:d1:04:17:09:f6:d0:7f:18:34:63:d4:3e:5f",
 				UsedBy: []CertificateUsedByRef{
@@ -1775,9 +1996,9 @@ func TestCertificateFromSchema(t *testing.T) {
 				Name:           "managed certificate",
 				Type:           "managed",
 				Certificate:    "-----BEGIN CERTIFICATE-----\n...",
-				Created:        mustParseTime(t, apiTimestampFormat, "2016-01-30T23:50:00+00:00"),
-				NotValidBefore: mustParseTime(t, apiTimestampFormat, "2016-01-30T23:51:00+00:00"),
-				NotValidAfter:  mustParseTime(t, apiTimestampFormat, "2016-01-30T23:55:00+00:00"),
+				Created:        mustParseTime(t, "2016-01-30T23:50:00+00:00"),
+				NotValidBefore: mustParseTime(t, "2016-01-30T23:51:00+00:00"),
+				NotValidAfter:  mustParseTime(t, "2016-01-30T23:55:00+00:00"),
 				DomainNames:    []string{"example.com", "webmail.example.com", "www.example.com"},
 				Fingerprint:    "03:c7:55:9b:2a:d1:04:17:09:f6:d0:7f:18:34:63:d4:3e:5f",
 				Status: &CertificateStatus{
@@ -1837,6 +2058,24 @@ func TestPricingFromSchema(t *testing.T) {
 				"type": "ipv4"
 			  }
 			],
+		"primary_ips": [
+			{
+				"prices": [
+				{
+					"location": "fsn1",
+					"price_hourly": {
+					"gross": "1.1900000000000000",
+					"net": "1.0000000000"
+					},
+					"price_monthly": {
+					"gross": "1.1900000000000000",
+					"net": "1.0000000000"
+					}
+				}
+				],
+				"type": "ipv4"
+			}
+		],
 		"traffic": {
 			"price_per_tb": {
 				"net": "1",
@@ -1953,6 +2192,35 @@ func TestPricingFromSchema(t *testing.T) {
 		}
 	}
 
+	if len(pricing.PrimaryIPs) != 1 {
+		t.Errorf("unexpected number of Primary IPs: %d", len(pricing.PrimaryIPs))
+	} else {
+		ip := pricing.PrimaryIPs[0]
+
+		if ip.Type != "ipv4" {
+			t.Errorf("unexpected .Type: %s", ip.Type)
+		}
+		if len(ip.Pricings) != 1 {
+			t.Errorf("unexpected number of prices: %d", len(ip.Pricings))
+		} else {
+			if ip.Pricings[0].Location != "fsn1" {
+				t.Errorf("unexpected Location: %v", ip.Pricings[0].Location)
+			}
+			if ip.Pricings[0].Monthly.Net != "1.0000000000" {
+				t.Errorf("unexpected Monthly.Net: %v", ip.Pricings[0].Monthly.Net)
+			}
+			if ip.Pricings[0].Monthly.Gross != "1.1900000000000000" {
+				t.Errorf("unexpected Monthly.Gross: %v", ip.Pricings[0].Monthly.Gross)
+			}
+			if ip.Pricings[0].Hourly.Net != "1.0000000000" {
+				t.Errorf("unexpected Hourly.Net: %v", ip.Pricings[0].Hourly.Net)
+			}
+			if ip.Pricings[0].Hourly.Gross != "1.1900000000000000" {
+				t.Errorf("unexpected Hourly.Gross: %v", ip.Pricings[0].Hourly.Gross)
+			}
+		}
+	}
+
 	if pricing.Volume.PerGBMonthly.Currency != "EUR" {
 		t.Errorf("unexpected Traffic.PerTB.Currency: %v", pricing.Volume.PerGBMonthly.Currency)
 	}
@@ -2096,7 +2364,7 @@ func TestLoadBalancerCreateOptsToSchema(t *testing.T) {
 				Algorithm: &schema.LoadBalancerCreateRequestAlgorithm{
 					Type: string(LoadBalancerAlgorithmTypeRoundRobin),
 				},
-				NetworkZone: String(string(NetworkZoneEUCentral)),
+				NetworkZone: Ptr(string(NetworkZoneEUCentral)),
 			},
 		},
 		"all set": {
@@ -2106,32 +2374,32 @@ func TestLoadBalancerCreateOptsToSchema(t *testing.T) {
 				Algorithm:        &LoadBalancerAlgorithm{Type: LoadBalancerAlgorithmTypeRoundRobin},
 				NetworkZone:      NetworkZoneEUCentral,
 				Labels:           map[string]string{"foo": "bar"},
-				PublicInterface:  Bool(true),
+				PublicInterface:  Ptr(true),
 				Network:          &Network{ID: 3},
 				Services: []LoadBalancerCreateOptsService{
 					{
 						Protocol:        LoadBalancerServiceProtocolHTTP,
-						DestinationPort: Int(80),
-						Proxyprotocol:   Bool(true),
+						DestinationPort: Ptr(80),
+						Proxyprotocol:   Ptr(true),
 						HTTP: &LoadBalancerCreateOptsServiceHTTP{
-							CookieName:     String("keks"),
-							CookieLifetime: Duration(5 * time.Minute),
-							RedirectHTTP:   Bool(true),
-							StickySessions: Bool(true),
+							CookieName:     Ptr("keks"),
+							CookieLifetime: Ptr(5 * time.Minute),
+							RedirectHTTP:   Ptr(true),
+							StickySessions: Ptr(true),
 							Certificates:   []*Certificate{{ID: 1}, {ID: 2}},
 						},
 						HealthCheck: &LoadBalancerCreateOptsServiceHealthCheck{
 							Protocol: LoadBalancerServiceProtocolHTTP,
-							Port:     Int(80),
-							Interval: Duration(5 * time.Second),
-							Timeout:  Duration(1 * time.Second),
-							Retries:  Int(3),
+							Port:     Ptr(80),
+							Interval: Ptr(5 * time.Second),
+							Timeout:  Ptr(1 * time.Second),
+							Retries:  Ptr(3),
 							HTTP: &LoadBalancerCreateOptsServiceHealthCheckHTTP{
-								Domain:      String("example.com"),
-								Path:        String("/health"),
-								Response:    String("ok"),
+								Domain:      Ptr("example.com"),
+								Path:        Ptr("/health"),
+								Response:    Ptr("ok"),
 								StatusCodes: []string{"2??", "3??"},
-								TLS:         Bool(true),
+								TLS:         Ptr(true),
 							},
 						},
 					},
@@ -2155,37 +2423,37 @@ func TestLoadBalancerCreateOptsToSchema(t *testing.T) {
 				Algorithm: &schema.LoadBalancerCreateRequestAlgorithm{
 					Type: string(LoadBalancerAlgorithmTypeRoundRobin),
 				},
-				NetworkZone: String(string(NetworkZoneEUCentral)),
+				NetworkZone: Ptr(string(NetworkZoneEUCentral)),
 				Labels: func() *map[string]string {
 					labels := map[string]string{"foo": "bar"}
 					return &labels
 				}(),
-				PublicInterface: Bool(true),
-				Network:         Int(3),
+				PublicInterface: Ptr(true),
+				Network:         Ptr(3),
 				Services: []schema.LoadBalancerCreateRequestService{
 					{
 						Protocol:        string(LoadBalancerServiceProtocolHTTP),
-						DestinationPort: Int(80),
-						Proxyprotocol:   Bool(true),
+						DestinationPort: Ptr(80),
+						Proxyprotocol:   Ptr(true),
 						HTTP: &schema.LoadBalancerCreateRequestServiceHTTP{
-							CookieName:     String("keks"),
-							CookieLifetime: Int(5 * 60),
-							RedirectHTTP:   Bool(true),
-							StickySessions: Bool(true),
-							Certificates:   intSlice([]int{1, 2}),
+							CookieName:     Ptr("keks"),
+							CookieLifetime: Ptr(5 * 60),
+							RedirectHTTP:   Ptr(true),
+							StickySessions: Ptr(true),
+							Certificates:   Ptr([]int{1, 2}),
 						},
 						HealthCheck: &schema.LoadBalancerCreateRequestServiceHealthCheck{
 							Protocol: string(LoadBalancerServiceProtocolHTTP),
-							Port:     Int(80),
-							Interval: Int(5),
-							Timeout:  Int(1),
-							Retries:  Int(3),
+							Port:     Ptr(80),
+							Interval: Ptr(5),
+							Timeout:  Ptr(1),
+							Retries:  Ptr(3),
 							HTTP: &schema.LoadBalancerCreateRequestServiceHealthCheckHTTP{
-								Domain:      String("example.com"),
-								Path:        String("/health"),
-								Response:    String("ok"),
-								StatusCodes: stringSlice([]string{"2??", "3??"}),
-								TLS:         Bool(true),
+								Domain:      Ptr("example.com"),
+								Path:        Ptr("/health"),
+								Response:    Ptr("ok"),
+								StatusCodes: Ptr([]string{"2??", "3??"}),
+								TLS:         Ptr(true),
 							},
 						},
 					},
@@ -2234,53 +2502,53 @@ func TestLoadBalancerAddServiceOptsToSchema(t *testing.T) {
 		"all set": {
 			Opts: LoadBalancerAddServiceOpts{
 				Protocol:        LoadBalancerServiceProtocolHTTP,
-				DestinationPort: Int(80),
-				Proxyprotocol:   Bool(true),
+				DestinationPort: Ptr(80),
+				Proxyprotocol:   Ptr(true),
 				HTTP: &LoadBalancerAddServiceOptsHTTP{
-					CookieName:     String("keks"),
-					CookieLifetime: Duration(5 * time.Minute),
-					RedirectHTTP:   Bool(true),
-					StickySessions: Bool(true),
+					CookieName:     Ptr("keks"),
+					CookieLifetime: Ptr(5 * time.Minute),
+					RedirectHTTP:   Ptr(true),
+					StickySessions: Ptr(true),
 					Certificates:   []*Certificate{{ID: 1}, {ID: 2}},
 				},
 				HealthCheck: &LoadBalancerAddServiceOptsHealthCheck{
 					Protocol: LoadBalancerServiceProtocolHTTP,
-					Port:     Int(80),
-					Interval: Duration(5 * time.Second),
-					Timeout:  Duration(1 * time.Second),
-					Retries:  Int(3),
+					Port:     Ptr(80),
+					Interval: Ptr(5 * time.Second),
+					Timeout:  Ptr(1 * time.Second),
+					Retries:  Ptr(3),
 					HTTP: &LoadBalancerAddServiceOptsHealthCheckHTTP{
-						Domain:      String("example.com"),
-						Path:        String("/health"),
-						Response:    String("ok"),
+						Domain:      Ptr("example.com"),
+						Path:        Ptr("/health"),
+						Response:    Ptr("ok"),
 						StatusCodes: []string{"2??", "3??"},
-						TLS:         Bool(true),
+						TLS:         Ptr(true),
 					},
 				},
 			},
 			Request: schema.LoadBalancerActionAddServiceRequest{
 				Protocol:        string(LoadBalancerServiceProtocolHTTP),
-				DestinationPort: Int(80),
-				Proxyprotocol:   Bool(true),
+				DestinationPort: Ptr(80),
+				Proxyprotocol:   Ptr(true),
 				HTTP: &schema.LoadBalancerActionAddServiceRequestHTTP{
-					CookieName:     String("keks"),
-					CookieLifetime: Int(5 * 60),
-					RedirectHTTP:   Bool(true),
-					StickySessions: Bool(true),
-					Certificates:   intSlice([]int{1, 2}),
+					CookieName:     Ptr("keks"),
+					CookieLifetime: Ptr(5 * 60),
+					RedirectHTTP:   Ptr(true),
+					StickySessions: Ptr(true),
+					Certificates:   Ptr([]int{1, 2}),
 				},
 				HealthCheck: &schema.LoadBalancerActionAddServiceRequestHealthCheck{
 					Protocol: string(LoadBalancerServiceProtocolHTTP),
-					Port:     Int(80),
-					Interval: Int(5),
-					Timeout:  Int(1),
-					Retries:  Int(3),
+					Port:     Ptr(80),
+					Interval: Ptr(5),
+					Timeout:  Ptr(1),
+					Retries:  Ptr(3),
 					HTTP: &schema.LoadBalancerActionAddServiceRequestHealthCheckHTTP{
-						Domain:      String("example.com"),
-						Path:        String("/health"),
-						Response:    String("ok"),
-						StatusCodes: stringSlice([]string{"2??", "3??"}),
-						TLS:         Bool(true),
+						Domain:      Ptr("example.com"),
+						Path:        Ptr("/health"),
+						Response:    Ptr("ok"),
+						StatusCodes: Ptr([]string{"2??", "3??"}),
+						TLS:         Ptr(true),
 					},
 				},
 			},
@@ -2288,26 +2556,26 @@ func TestLoadBalancerAddServiceOptsToSchema(t *testing.T) {
 		"no health check": {
 			Opts: LoadBalancerAddServiceOpts{
 				Protocol:        LoadBalancerServiceProtocolHTTP,
-				DestinationPort: Int(80),
-				Proxyprotocol:   Bool(true),
+				DestinationPort: Ptr(80),
+				Proxyprotocol:   Ptr(true),
 				HTTP: &LoadBalancerAddServiceOptsHTTP{
-					CookieName:     String("keks"),
-					CookieLifetime: Duration(5 * time.Minute),
-					RedirectHTTP:   Bool(true),
-					StickySessions: Bool(true),
+					CookieName:     Ptr("keks"),
+					CookieLifetime: Ptr(5 * time.Minute),
+					RedirectHTTP:   Ptr(true),
+					StickySessions: Ptr(true),
 					Certificates:   []*Certificate{{ID: 1}, {ID: 2}},
 				},
 			},
 			Request: schema.LoadBalancerActionAddServiceRequest{
 				Protocol:        string(LoadBalancerServiceProtocolHTTP),
-				DestinationPort: Int(80),
-				Proxyprotocol:   Bool(true),
+				DestinationPort: Ptr(80),
+				Proxyprotocol:   Ptr(true),
 				HTTP: &schema.LoadBalancerActionAddServiceRequestHTTP{
-					CookieName:     String("keks"),
-					CookieLifetime: Int(5 * 60),
-					RedirectHTTP:   Bool(true),
-					StickySessions: Bool(true),
-					Certificates:   intSlice([]int{1, 2}),
+					CookieName:     Ptr("keks"),
+					CookieLifetime: Ptr(5 * 60),
+					RedirectHTTP:   Ptr(true),
+					StickySessions: Ptr(true),
+					Certificates:   Ptr([]int{1, 2}),
 				},
 				HealthCheck: nil,
 			},
@@ -2336,53 +2604,53 @@ func TestLoadBalancerUpdateServiceOptsToSchema(t *testing.T) {
 		"all set": {
 			Opts: LoadBalancerUpdateServiceOpts{
 				Protocol:        LoadBalancerServiceProtocolHTTP,
-				DestinationPort: Int(80),
-				Proxyprotocol:   Bool(true),
+				DestinationPort: Ptr(80),
+				Proxyprotocol:   Ptr(true),
 				HTTP: &LoadBalancerUpdateServiceOptsHTTP{
-					CookieName:     String("keks"),
-					CookieLifetime: Duration(5 * time.Minute),
-					RedirectHTTP:   Bool(true),
-					StickySessions: Bool(true),
+					CookieName:     Ptr("keks"),
+					CookieLifetime: Ptr(5 * time.Minute),
+					RedirectHTTP:   Ptr(true),
+					StickySessions: Ptr(true),
 					Certificates:   []*Certificate{{ID: 1}, {ID: 2}},
 				},
 				HealthCheck: &LoadBalancerUpdateServiceOptsHealthCheck{
 					Protocol: LoadBalancerServiceProtocolHTTP,
-					Port:     Int(80),
-					Interval: Duration(5 * time.Second),
-					Timeout:  Duration(1 * time.Second),
-					Retries:  Int(3),
+					Port:     Ptr(80),
+					Interval: Ptr(5 * time.Second),
+					Timeout:  Ptr(1 * time.Second),
+					Retries:  Ptr(3),
 					HTTP: &LoadBalancerUpdateServiceOptsHealthCheckHTTP{
-						Domain:      String("example.com"),
-						Path:        String("/health"),
-						Response:    String("ok"),
+						Domain:      Ptr("example.com"),
+						Path:        Ptr("/health"),
+						Response:    Ptr("ok"),
 						StatusCodes: []string{"2??", "3??"},
-						TLS:         Bool(true),
+						TLS:         Ptr(true),
 					},
 				},
 			},
 			Request: schema.LoadBalancerActionUpdateServiceRequest{
-				Protocol:        String(string(LoadBalancerServiceProtocolHTTP)),
-				DestinationPort: Int(80),
-				Proxyprotocol:   Bool(true),
+				Protocol:        Ptr(string(LoadBalancerServiceProtocolHTTP)),
+				DestinationPort: Ptr(80),
+				Proxyprotocol:   Ptr(true),
 				HTTP: &schema.LoadBalancerActionUpdateServiceRequestHTTP{
-					CookieName:     String("keks"),
-					CookieLifetime: Int(5 * 60),
-					RedirectHTTP:   Bool(true),
-					StickySessions: Bool(true),
-					Certificates:   intSlice([]int{1, 2}),
+					CookieName:     Ptr("keks"),
+					CookieLifetime: Ptr(5 * 60),
+					RedirectHTTP:   Ptr(true),
+					StickySessions: Ptr(true),
+					Certificates:   Ptr([]int{1, 2}),
 				},
 				HealthCheck: &schema.LoadBalancerActionUpdateServiceRequestHealthCheck{
-					Protocol: String(string(LoadBalancerServiceProtocolHTTP)),
-					Port:     Int(80),
-					Interval: Int(5),
-					Timeout:  Int(1),
-					Retries:  Int(3),
+					Protocol: Ptr(string(LoadBalancerServiceProtocolHTTP)),
+					Port:     Ptr(80),
+					Interval: Ptr(5),
+					Timeout:  Ptr(1),
+					Retries:  Ptr(3),
 					HTTP: &schema.LoadBalancerActionUpdateServiceRequestHealthCheckHTTP{
-						Domain:      String("example.com"),
-						Path:        String("/health"),
-						Response:    String("ok"),
-						StatusCodes: stringSlice([]string{"2??", "3??"}),
-						TLS:         Bool(true),
+						Domain:      Ptr("example.com"),
+						Path:        Ptr("/health"),
+						Response:    Ptr("ok"),
+						StatusCodes: Ptr([]string{"2??", "3??"}),
+						TLS:         Ptr(true),
 					},
 				},
 			},
@@ -2390,26 +2658,26 @@ func TestLoadBalancerUpdateServiceOptsToSchema(t *testing.T) {
 		"no health check": {
 			Opts: LoadBalancerUpdateServiceOpts{
 				Protocol:        LoadBalancerServiceProtocolHTTP,
-				DestinationPort: Int(80),
-				Proxyprotocol:   Bool(true),
+				DestinationPort: Ptr(80),
+				Proxyprotocol:   Ptr(true),
 				HTTP: &LoadBalancerUpdateServiceOptsHTTP{
-					CookieName:     String("keks"),
-					CookieLifetime: Duration(5 * time.Minute),
-					RedirectHTTP:   Bool(true),
-					StickySessions: Bool(true),
+					CookieName:     Ptr("keks"),
+					CookieLifetime: Ptr(5 * time.Minute),
+					RedirectHTTP:   Ptr(true),
+					StickySessions: Ptr(true),
 					Certificates:   []*Certificate{{ID: 1}, {ID: 2}},
 				},
 			},
 			Request: schema.LoadBalancerActionUpdateServiceRequest{
-				Protocol:        String(string(LoadBalancerServiceProtocolHTTP)),
-				DestinationPort: Int(80),
-				Proxyprotocol:   Bool(true),
+				Protocol:        Ptr(string(LoadBalancerServiceProtocolHTTP)),
+				DestinationPort: Ptr(80),
+				Proxyprotocol:   Ptr(true),
 				HTTP: &schema.LoadBalancerActionUpdateServiceRequestHTTP{
-					CookieName:     String("keks"),
-					CookieLifetime: Int(5 * 60),
-					RedirectHTTP:   Bool(true),
-					StickySessions: Bool(true),
-					Certificates:   intSlice([]int{1, 2}),
+					CookieName:     Ptr("keks"),
+					CookieLifetime: Ptr(5 * 60),
+					RedirectHTTP:   Ptr(true),
+					StickySessions: Ptr(true),
+					Certificates:   Ptr([]int{1, 2}),
 				},
 				HealthCheck: nil,
 			},
@@ -2438,8 +2706,8 @@ func TestServerMetricsFromSchema(t *testing.T) {
 			respFn: func() *schema.ServerGetMetricsResponse {
 				var resp schema.ServerGetMetricsResponse
 
-				resp.Metrics.Start = mustParseTime(t, time.RFC3339, "2017-01-01T00:00:00Z")
-				resp.Metrics.End = mustParseTime(t, time.RFC3339, "2017-01-01T23:00:00Z")
+				resp.Metrics.Start = mustParseTime(t, "2017-01-01T00:00:00Z")
+				resp.Metrics.End = mustParseTime(t, "2017-01-01T23:00:00Z")
 				resp.Metrics.TimeSeries = map[string]schema.ServerTimeSeriesVals{
 					"cpu": {
 						Values: []interface{}{"some value"},
@@ -2455,8 +2723,8 @@ func TestServerMetricsFromSchema(t *testing.T) {
 			respFn: func() *schema.ServerGetMetricsResponse {
 				var resp schema.ServerGetMetricsResponse
 
-				resp.Metrics.Start = mustParseTime(t, time.RFC3339, "2017-01-01T00:00:00Z")
-				resp.Metrics.End = mustParseTime(t, time.RFC3339, "2017-01-01T23:00:00Z")
+				resp.Metrics.Start = mustParseTime(t, "2017-01-01T00:00:00Z")
+				resp.Metrics.End = mustParseTime(t, "2017-01-01T23:00:00Z")
 				resp.Metrics.TimeSeries = map[string]schema.ServerTimeSeriesVals{
 					"cpu": {
 						Values: []interface{}{
@@ -2474,8 +2742,8 @@ func TestServerMetricsFromSchema(t *testing.T) {
 			respFn: func() *schema.ServerGetMetricsResponse {
 				var resp schema.ServerGetMetricsResponse
 
-				resp.Metrics.Start = mustParseTime(t, time.RFC3339, "2017-01-01T00:00:00Z")
-				resp.Metrics.End = mustParseTime(t, time.RFC3339, "2017-01-01T23:00:00Z")
+				resp.Metrics.Start = mustParseTime(t, "2017-01-01T00:00:00Z")
+				resp.Metrics.End = mustParseTime(t, "2017-01-01T23:00:00Z")
 				resp.Metrics.TimeSeries = map[string]schema.ServerTimeSeriesVals{
 					"cpu": {
 						Values: []interface{}{
@@ -2493,8 +2761,8 @@ func TestServerMetricsFromSchema(t *testing.T) {
 			respFn: func() *schema.ServerGetMetricsResponse {
 				var resp schema.ServerGetMetricsResponse
 
-				resp.Metrics.Start = mustParseTime(t, time.RFC3339, "2017-01-01T00:00:00Z")
-				resp.Metrics.End = mustParseTime(t, time.RFC3339, "2017-01-01T23:00:00Z")
+				resp.Metrics.Start = mustParseTime(t, "2017-01-01T00:00:00Z")
+				resp.Metrics.End = mustParseTime(t, "2017-01-01T23:00:00Z")
 				resp.Metrics.TimeSeries = map[string]schema.ServerTimeSeriesVals{
 					"cpu": {
 						Values: []interface{}{
@@ -2512,8 +2780,8 @@ func TestServerMetricsFromSchema(t *testing.T) {
 			respFn: func() *schema.ServerGetMetricsResponse {
 				var resp schema.ServerGetMetricsResponse
 
-				resp.Metrics.Start = mustParseTime(t, time.RFC3339, "2017-01-01T00:00:00Z")
-				resp.Metrics.End = mustParseTime(t, time.RFC3339, "2017-01-01T23:00:00Z")
+				resp.Metrics.Start = mustParseTime(t, "2017-01-01T00:00:00Z")
+				resp.Metrics.End = mustParseTime(t, "2017-01-01T23:00:00Z")
 				resp.Metrics.TimeSeries = map[string]schema.ServerTimeSeriesVals{
 					"cpu": {
 						Values: []interface{}{
@@ -2550,8 +2818,8 @@ func TestServerMetricsFromSchema(t *testing.T) {
 				return &resp
 			},
 			expected: &ServerMetrics{
-				Start: mustParseTime(t, time.RFC3339, "2017-01-01T00:00:00Z"),
-				End:   mustParseTime(t, time.RFC3339, "2017-01-01T23:00:00Z"),
+				Start: mustParseTime(t, "2017-01-01T00:00:00Z"),
+				End:   mustParseTime(t, "2017-01-01T23:00:00Z"),
 				TimeSeries: map[string][]ServerMetricsValue{
 					"cpu": {
 						{Timestamp: 1435781470.622, Value: "42"},
@@ -2608,8 +2876,8 @@ func TestLoadBalancerMetricsFromSchema(t *testing.T) {
 			respFn: func() *schema.LoadBalancerGetMetricsResponse {
 				var resp schema.LoadBalancerGetMetricsResponse
 
-				resp.Metrics.Start = mustParseTime(t, time.RFC3339, "2017-01-01T00:00:00Z")
-				resp.Metrics.End = mustParseTime(t, time.RFC3339, "2017-01-01T23:00:00Z")
+				resp.Metrics.Start = mustParseTime(t, "2017-01-01T00:00:00Z")
+				resp.Metrics.End = mustParseTime(t, "2017-01-01T23:00:00Z")
 				resp.Metrics.TimeSeries = map[string]schema.LoadBalancerTimeSeriesVals{
 					"open_connections": {
 						Values: []interface{}{"some value"},
@@ -2625,8 +2893,8 @@ func TestLoadBalancerMetricsFromSchema(t *testing.T) {
 			respFn: func() *schema.LoadBalancerGetMetricsResponse {
 				var resp schema.LoadBalancerGetMetricsResponse
 
-				resp.Metrics.Start = mustParseTime(t, time.RFC3339, "2017-01-01T00:00:00Z")
-				resp.Metrics.End = mustParseTime(t, time.RFC3339, "2017-01-01T23:00:00Z")
+				resp.Metrics.Start = mustParseTime(t, "2017-01-01T00:00:00Z")
+				resp.Metrics.End = mustParseTime(t, "2017-01-01T23:00:00Z")
 				resp.Metrics.TimeSeries = map[string]schema.LoadBalancerTimeSeriesVals{
 					"open_connections": {
 						Values: []interface{}{
@@ -2644,8 +2912,8 @@ func TestLoadBalancerMetricsFromSchema(t *testing.T) {
 			respFn: func() *schema.LoadBalancerGetMetricsResponse {
 				var resp schema.LoadBalancerGetMetricsResponse
 
-				resp.Metrics.Start = mustParseTime(t, time.RFC3339, "2017-01-01T00:00:00Z")
-				resp.Metrics.End = mustParseTime(t, time.RFC3339, "2017-01-01T23:00:00Z")
+				resp.Metrics.Start = mustParseTime(t, "2017-01-01T00:00:00Z")
+				resp.Metrics.End = mustParseTime(t, "2017-01-01T23:00:00Z")
 				resp.Metrics.TimeSeries = map[string]schema.LoadBalancerTimeSeriesVals{
 					"open_connections": {
 						Values: []interface{}{
@@ -2663,8 +2931,8 @@ func TestLoadBalancerMetricsFromSchema(t *testing.T) {
 			respFn: func() *schema.LoadBalancerGetMetricsResponse {
 				var resp schema.LoadBalancerGetMetricsResponse
 
-				resp.Metrics.Start = mustParseTime(t, time.RFC3339, "2017-01-01T00:00:00Z")
-				resp.Metrics.End = mustParseTime(t, time.RFC3339, "2017-01-01T23:00:00Z")
+				resp.Metrics.Start = mustParseTime(t, "2017-01-01T00:00:00Z")
+				resp.Metrics.End = mustParseTime(t, "2017-01-01T23:00:00Z")
 				resp.Metrics.TimeSeries = map[string]schema.LoadBalancerTimeSeriesVals{
 					"open_connections": {
 						Values: []interface{}{
@@ -2682,8 +2950,8 @@ func TestLoadBalancerMetricsFromSchema(t *testing.T) {
 			respFn: func() *schema.LoadBalancerGetMetricsResponse {
 				var resp schema.LoadBalancerGetMetricsResponse
 
-				resp.Metrics.Start = mustParseTime(t, time.RFC3339, "2017-01-01T00:00:00Z")
-				resp.Metrics.End = mustParseTime(t, time.RFC3339, "2017-01-01T23:00:00Z")
+				resp.Metrics.Start = mustParseTime(t, "2017-01-01T00:00:00Z")
+				resp.Metrics.End = mustParseTime(t, "2017-01-01T23:00:00Z")
 				resp.Metrics.TimeSeries = map[string]schema.LoadBalancerTimeSeriesVals{
 					"open_connections": {
 						Values: []interface{}{
@@ -2720,8 +2988,8 @@ func TestLoadBalancerMetricsFromSchema(t *testing.T) {
 				return &resp
 			},
 			expected: &LoadBalancerMetrics{
-				Start: mustParseTime(t, time.RFC3339, "2017-01-01T00:00:00Z"),
-				End:   mustParseTime(t, time.RFC3339, "2017-01-01T23:00:00Z"),
+				Start: mustParseTime(t, "2017-01-01T00:00:00Z"),
+				End:   mustParseTime(t, "2017-01-01T23:00:00Z"),
 				TimeSeries: map[string][]LoadBalancerMetricsValue{
 					"open_connections": {
 						{Timestamp: 1435781470.622, Value: "42"},
@@ -2905,3 +3173,43 @@ func TestPlacementGroupFromSchema(t *testing.T) {
 		t.Errorf("unexpected Type %s", placementGroup.Type)
 	}
 }
+
+func TestDeprecationFromSchema(t *testing.T) {
+	t.Run("Deprecated Resource", func(t *testing.T) {
+		data := []byte(`{
+			"deprecation": {
+				"announced": "2023-06-01T00:00:00+00:00",
+				"unavailable_after": "2023-09-01T00:00:00+00:00"
+			}
+		}`)
+
+		var d schema.DeprecatableResource
+		if err := json.Unmarshal(data, &d); err != nil {
+			t.Fatal(err)
+		}
+		deprecationInfo := DeprecationFromSchema(d.Deprecation)
+		if deprecationInfo == nil {
+			t.Fatal("unexpected nil DeprecationInfo")
+		}
+		if deprecationInfo.Announced != mustParseTime(t, "2023-06-01T00:00:00+00:00") {
+			t.Errorf("unexpected anounce time %s", deprecationInfo.Announced)
+		}
+		if deprecationInfo.UnavailableAfter != mustParseTime(t, "2023-09-01T00:00:00+00:00") {
+			t.Errorf("unexpected unavailable after time %s", deprecationInfo.UnavailableAfter)
+		}
+	})
+
+	t.Run("Not-deprecated Resource", func(t *testing.T) {
+		data := []byte(`{
+			"deprecation": null
+		}`)
+		var d schema.DeprecatableResource
+		if err := json.Unmarshal(data, &d); err != nil {
+			t.Fatal(err)
+		}
+		deprecationInfo := DeprecationFromSchema(d.Deprecation)
+		if deprecationInfo != nil {
+			t.Fatal("unexpected non-nil DeprecationInfo", deprecationInfo)
+		}
+	})
+}
